"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getMermaidDSL = exports.getNomnomlDSL = exports.parseProject = exports.createDiagram = void 0;
const io_1 = require("./io");
const parser_1 = require("./parser/parser");
const emitter_1 = require("./emitter");
const nomnoml_template_1 = require("./renderer/nomnoml-template");
const tsuml2_settings_1 = require("./tsuml2-settings");
const chalk_1 = __importDefault(require("chalk"));
const fs = __importStar(require("fs"));
const parser_2 = require("./parser");
const mermaid_template_1 = require("./renderer/mermaid-template");
function createDiagram(settings) {
    // parse
    const declarations = parseProject(settings.tsconfig, settings.glob);
    if (declarations.length === 0) {
        console.log(chalk_1.default.red("\nno declarations found! tsconfig: " + settings.tsconfig, " glob: " + settings.glob));
        return;
    }
    // emit
    createMermaidDSL(declarations, settings);
    createNomnomlSVG(declarations, settings);
}
exports.createDiagram = createDiagram;
/**
 * parse a typescript project
 * @param tsConfigPath path to a tsconfig.json file
 * @param pattern the glob pattern defining a scope for typescript files to include (i.e.: a subfolder of the project)
 * @returns
 */
function parseProject(tsConfigPath, pattern) {
    const ast = (0, parser_1.getAst)(tsConfigPath, pattern);
    const files = ast.getSourceFiles();
    // parser
    console.log(chalk_1.default.yellow("parsing source files:"));
    const declarations = files.map(f => {
        const classes = f.getClasses();
        const interfaces = f.getInterfaces();
        const enums = f.getEnums();
        const types = f.getTypeAliases();
        const path = f.getFilePath();
        console.log(chalk_1.default.yellow(path));
        const classDeclarations = classes.map(parser_1.parseClasses);
        const interfaceDeclarations = interfaces.map(parser_1.parseInterfaces);
        return {
            fileName: path,
            classes: classDeclarations,
            interfaces: interfaceDeclarations,
            types: types.map(parser_1.parseTypes).filter(t => t !== undefined),
            enums: enums.map(parser_1.parseEnum),
            heritageClauses: [
                ...classDeclarations.filter(decl => decl.heritageClauses.length > 0).map(decl => decl.heritageClauses),
                ...interfaceDeclarations.filter(decl => decl.heritageClauses.length > 0).map(decl => decl.heritageClauses)
            ]
        };
    });
    if (tsuml2_settings_1.SETTINGS.memberAssociations) {
        (0, parser_2.parseAssociations)(declarations);
    }
    return declarations;
}
exports.parseProject = parseProject;
function createNomnomlSVG(declarations, settings) {
    const outDSL = settings.outDsl;
    const outFile = settings.outFile;
    if (outDSL === "" && outFile === "") {
        return;
    }
    const dsl = getNomnomlDSL(declarations, settings);
    if (outDSL !== "") {
        writeDsl(dsl, outDSL, 'nomnoml');
    }
    if (outFile === "") {
        return;
    }
    //render
    console.log(chalk_1.default.yellow("\nrender to svg"));
    let svg = (0, io_1.renderNomnomlSVG)(dsl);
    if (settings.typeLinks) {
        console.log(chalk_1.default.yellow("adding type links to svg"));
        svg = (0, emitter_1.postProcessSvg)(svg, outFile, declarations);
    }
    console.log(chalk_1.default.green("\nwriting SVG"));
    fs.writeFile(outFile, svg, (err) => {
        if (err) {
            console.log(chalk_1.default.redBright("Error writing file: " + err));
        }
    });
    return svg;
}
function createMermaidDSL(declarations, settings) {
    if (!settings.outMermaidDsl) {
        return;
    }
    const dsl = getMermaidDSL(declarations, settings);
    writeDsl(dsl, settings.outMermaidDsl, 'mermaid');
    return dsl;
}
/**
 * get the nomnoml DSL representing the class diagram as a string
 * @param declarations
 * @param settings
 * @returns
 */
function getNomnomlDSL(declarations, settings) {
    console.log(chalk_1.default.yellow("\nemitting nomnoml declarations:"));
    return getNomnomlDSLHeader(settings) + (0, emitter_1.emit)(declarations, new emitter_1.Emitter(nomnoml_template_1.nomnomlTemplate));
}
exports.getNomnomlDSL = getNomnomlDSL;
/**
 * get the mermaid DSL representing the class diagram as a string
 * @param declarations
 * @param settings
 * @returns
 */
function getMermaidDSL(declarations, settings) {
    console.log(chalk_1.default.yellow("\nemitting mermaid declarations:"));
    return getMermaidDSLHeader(settings) + (0, emitter_1.emit)(declarations, new emitter_1.Emitter(mermaid_template_1.mermaidTemplate));
}
exports.getMermaidDSL = getMermaidDSL;
function getNomnomlDSLHeader(settings) {
    return '#.interface: fill=lightblue\n' +
        '#.enumeration: fill=lightgreen\n' +
        '#.type: fill=lightgray\n' +
        settings.nomnoml.join("\n");
}
function getMermaidDSLHeader(settings) {
    return '\nclassDiagram\n';
}
function writeDsl(dsl, fileName, dslType) {
    console.log(chalk_1.default.green(`\nwriting ${dslType} DSL`));
    fs.writeFile(fileName, dsl, (err) => {
        if (err) {
            console.log(chalk_1.default.redBright(`Error writing ${dslType} DSL file: ${err}`));
        }
    });
}
//# sourceMappingURL=index.js.map